from os import chmod
import yaml
from .python_util import execute
from .devops_build import DevopsBuild
from .credential import gopass_field_from_path, gopass_password_from_path

def add_c4k_mixin_config(config,
                         c4k_module_name,
                         c4k_config_dict,
                         c4k_auth_dict,
                         grafana_cloud_user=None,
                         grafana_cloud_password=None,
                         grafana_cloud_url='https://prometheus-prod-01-eu-west-0.grafana.net/api/prom/push'):
    if not grafana_cloud_user:
        grafana_cloud_user = gopass_field_from_path(
            'server/meissa/grafana-cloud', 'grafana-cloud-user')
    if not grafana_cloud_password:
        grafana_cloud_password = gopass_password_from_path(
            'server/meissa/grafana-cloud')
    c4k_auth_dict.update({'mon-auth': {
        'grafana-cloud-user': grafana_cloud_user,
        'grafana-cloud-password': grafana_cloud_password
    }})
    c4k_config_dict.update({'mon-cfg': {
        'grafana-cloud-url': grafana_cloud_url
    }})
    config.update({'C4kMixin': {'Config': c4k_config_dict,
                                'Auth': c4k_auth_dict,
                                'Name': c4k_module_name}})
    return config


class C4kMixin(DevopsBuild):
    def __init__(self, project, config):
        super().__init__(project, config)
        self.c4k_mixin_config = config['C4kMixin']['Config']
        self.c4k_mixin_auth = config['C4kMixin']['Auth']
        self.c4k_module_name = config['C4kMixin']['Name']
        tmp = self.c4k_mixin_config['mon-cfg']
        tmp.update({'cluster-name': self.c4k_module_name,
                    'cluster-stage': self.stage})
        self.c4k_mixin_config.update({'mon-cfg': tmp})

    def write_c4k_config(self):
        fqdn = self.get('fqdn')
        self.c4k_mixin_config.update({'fqdn': fqdn})
        with open(self.build_path() + '/out_c4k_config.yaml', 'w', encoding="utf-8") as output_file:
            yaml.dump(self.c4k_mixin_config, output_file)

    def write_c4k_auth(self):
        with open(self.build_path() + '/out_c4k_auth.yaml', 'w', encoding="utf-8") as output_file:
            yaml.dump(self.c4k_mixin_auth, output_file)
        chmod(self.build_path() + '/out_c4k_auth.yaml', 0o600)

    def c4k_apply(self, dry_run=False):
        cmd = f'c4k-{self.c4k_module_name}-standalone.jar {self.build_path()}/out_c4k_config.yaml {self.build_path()}/out_c4k_auth.yaml > {self.build_path()}/out_{self.c4k_module_name}.yaml'
        output = ''
        if dry_run:
            print(cmd)
        else:
            output = execute(cmd, True)
            print(output)
        return output



from string import Template
import deprecation
from .python_util import execute_live
from .devops_build import DevopsBuild


CONFIG_BASE = """
fqdn: $fqdn
"""
CONFIG_IPV4 = """node:
  ipv4: $ipv4
"""
CONFIG_IPV6 = """  ipv6: $ipv6
"""
CONFIG_CERTMANAGER = """certmanager:
  email: $letsencrypt_email
  letsencryptEndpoint: $letsencrypt_endpoint
"""
CONFIG_ECHO = """echo: $echo
"""


def add_provs_k3s_mixin_config(config,
                               provision_user='root',
                               echo=None,
                               k3s_config_template=None,
                               letsencrypt_email=None,
                               letsencrypt_endpoint=None,
                               fqdn=None,
                               ipv4=None,
                               ipv6=None,
                               app_filename_to_provision=None):
    template_text = k3s_config_template
    if template_text is None:
        template_text = CONFIG_BASE
        if letsencrypt_endpoint is not None:
            template_text += CONFIG_CERTMANAGER
        if echo is not None:
            template_text += CONFIG_ECHO
        if ipv4  is not None:
            template_text += CONFIG_IPV4
        if ipv6  is not None:
            template_text += CONFIG_IPV6

    config.update({'ProvsK3sMixin':
                   {'fqdn': fqdn,
                    'provision_user': provision_user,
                    'ipv4': ipv4,
                    'ipv6': ipv6,
                    'letsencrypt_email': letsencrypt_email,
                    'letsencrypt_endpoint': letsencrypt_endpoint,
                    'echo': echo,
                    'k3s_config_template': template_text,
                    'app_filename_to_provision': app_filename_to_provision}})
    return config


class ProvsK3sMixin(DevopsBuild):

    def __init__(self, project, config):
        super().__init__(project, config)
        provs_k3s_mixin_config = config['ProvsK3sMixin']
        self.fqdn = provs_k3s_mixin_config['fqdn']
        self.put('fqdn', self.fqdn)
        self.provision_user = provs_k3s_mixin_config['provision_user']
        self.put('provision_user', self.provision_user)
        self.ipv4 = provs_k3s_mixin_config['ipv4']
        self.put('ipv4', self.ipv4)
        self.ipv6 = provs_k3s_mixin_config['ipv6']
        self.put('ipv6', self.ipv6)
        self.letsencrypt_email = provs_k3s_mixin_config['letsencrypt_email']
        self.put('letsencrypt_email', self.letsencrypt_email)
        self.letsencrypt_endpoint = provs_k3s_mixin_config['letsencrypt_endpoint']
        self.put('letsencrypt_endpoint', self.letsencrypt_endpoint)
        self.echo = provs_k3s_mixin_config['echo']
        self.put('echo', self.echo)
        self.k3s_config_template_text = provs_k3s_mixin_config['k3s_config_template']
        self.k3s_config_template = Template(
            provs_k3s_mixin_config['k3s_config_template'])
        self.put('k3s_config_template', self.k3s_config_template)
        self.app_filename_to_provision = provs_k3s_mixin_config['app_filename_to_provision']
        self.put('app_filename_to_provision', self.app_filename_to_provision)

    def update_runtime_config(self, fqdn, ipv4, ipv6=None):
        self.fqdn = fqdn
        self.put('fqdn', fqdn)
        self.ipv4 = ipv4
        self.put('ipv4', ipv4)
        self.ipv6 = ipv6
        self.put('ipv6', ipv6)
        template_text = self.k3s_config_template_text
        if ipv4  is not None:
            template_text += CONFIG_IPV4
        if ipv6  is not None:
            template_text += CONFIG_IPV6
        self.k3s_config_template_text = template_text
        self.put('k3s_config_template_text', template_text)
        template = Template(template_text)
        self.k3s_config_template = template
        self.put('k3s_config_template', template)

    def write_provs_config(self):
        substitutes = self.get_keys(['fqdn', 'ipv4', 'ipv6', 'letsencrypt_email',
                                     'letsencrypt_endpoint', 'echo'])
        with open(self.build_path() + '/out_k3sServerConfig.yaml', "w", encoding="utf-8") as output_file:
            output_file.write(self.k3s_config_template.substitute(substitutes))

    @deprecation.deprecated(deprecated_in="3.1")
    def provs_server(self, dry_run=False):
        self.provs_apply(dry_run)

    def provs_apply(self, dry_run=False):
        cmd = ['provs-server.jar', 'k3s', self.provision_user + '@' + self.fqdn, '-c',
               self.build_path() + '/out_k3sServerConfig.yaml',
               '-a', self.build_path() + '/' + self.app_filename_to_provision]
        if dry_run:
            print(" ".join(cmd))
        else:
            execute_live(cmd)



from .devops_terraform_build import DevopsTerraformBuild, create_devops_terraform_build_config


def create_digitalocean_terraform_build_config(stage,
                                               project_root_path,
                                               module,
                                               additional_vars,
                                               do_api_key,
                                               do_spaces_access_id,
                                               do_spaces_secret_key,
                                               build_dir_name='target',
                                               output_json_name=None,
                                               use_workspace=True,
                                               use_package_common_files=True,
                                               build_commons_path=None,
                                               terraform_build_commons_dir_name='terraform',
                                               debug_print_terraform_command=False,
                                               additional_tfvar_files=None,
                                               terraform_semantic_version="1.0.8",
                                               ):
    if not additional_tfvar_files:
        additional_tfvar_files = []
    config = create_devops_terraform_build_config(stage,
                                                  project_root_path,
                                                  module,
                                                  additional_vars,
                                                  build_dir_name,
                                                  output_json_name,
                                                  use_workspace,
                                                  use_package_common_files,
                                                  build_commons_path,
                                                  terraform_build_commons_dir_name,
                                                  debug_print_terraform_command,
                                                  additional_tfvar_files,
                                                  terraform_semantic_version)
    config.update({'DigitaloceanTerraformBuild':
                   {'do_api_key': do_api_key,
                    'do_spaces_access_id': do_spaces_access_id,
                    'do_spaces_secret_key': do_spaces_secret_key}})
    return config


class DigitaloceanTerraformBuild(DevopsTerraformBuild):

    def __init__(self, project, config):
        super().__init__(project, config)
        do_mixin_config = config['DigitaloceanTerraformBuild']
        self.do_api_key = do_mixin_config['do_api_key']
        self.do_spaces_access_id = do_mixin_config['do_spaces_access_id']
        self.do_spaces_secret_key = do_mixin_config['do_spaces_secret_key']

    def project_vars(self):
        ret = super().project_vars()
        ret['do_api_key'] = self.do_api_key
        ret['do_spaces_access_id'] = self.do_spaces_access_id
        ret['do_spaces_secret_key'] = self.do_spaces_secret_key
        return ret

    def copy_build_resources_from_package(self):
        super().copy_build_resources_from_package()
        self.copy_build_resource_file_from_package('provider_registry.tf')
        self.copy_build_resource_file_from_package('do_provider.tf')
        self.copy_build_resource_file_from_package('do_mixin_vars.tf')



from subprocess import run, CalledProcessError
from .python_util import filter_none

def create_devops_build_config(stage, project_root_path, module,
                               build_dir_name='target'):
    return {'stage': stage,
            'project_root_path': project_root_path,
            'module': module,
            'build_dir_name': build_dir_name}

def get_devops_build(project):
    return project.get_property('devops_build')

def get_tag_from_latest_commit():
    try:
        value = run('git describe --abbrev=0 --tags --exact-match', shell=True,
            capture_output=True, check=True)
        return value.stdout.decode('UTF-8').rstrip()
    except CalledProcessError:
        return None

class DevopsBuild:

    def __init__(self, project, config):
        #deprecate stage
        self.stage = config['stage']
        self.project_root_path = config['project_root_path']
        self.module = config['module']
        self.build_dir_name = config['build_dir_name']
        self.stack = {}
        self.project = project
        project.set_property('devops_build', self)

    def name(self):
        return self.project.name

    def build_path(self):
        mylist = [self.project_root_path,
                  self.build_dir_name,
                  self.name(),
                  self.module]
        return '/'.join(filter_none(mylist))

    def initialize_build_dir(self):
        run('rm -rf ' + self.build_path(), shell=True, check=True)
        run('mkdir -p ' + self.build_path(), shell=True, check=True)

    def put(self, key, value):
        self.stack[key] = value

    def get(self, key):
        return self.stack[key]

    def get_keys(self, keys):
        result = {}
        for key in keys:
            result[key] = self.get(key)
        return result



from .python_util import execute

def gopass_field_from_path (path, field):
    credential = None
    if path and field:
        print('get field for: ' + path + ', ' + field)
        credential = execute(['gopass', 'show', path, field])
    return credential

def gopass_password_from_path (path):
    credential = None
    if path:
        print('get password for: ' + path)
        credential = execute(['gopass', 'show', '--password', path])
    return credential
